@@ -15,7 +15,9 @@ from contextlib import contextmanager
 
 import pytz
 from celery import current_app as current_celery_app
+from elasticsearch import VERSION as ES_VERSION
 from elasticsearch.helpers import bulk
+from elasticsearch.helpers import expand_action as default_expand_action
 from flask import current_app
 from invenio_records.api import Record
 from invenio_search import current_search_client
@@ -26,7 +28,7 @@ from sqlalchemy.orm.exc import NoResultFound
 
 from .proxies import current_record_to_index
 from .signals import before_record_index
-from .utils import es_bulk_param_compatibility
+from .utils import _es7_expand_action
 
 
 class Producer(KombuProducer):
@@ -48,6 +50,8 @@ class RecordIndexer(object):
     these requests in bulk.
     """
 
+    record_cls = Record
+
     def __init__(self, search_client=None, exchange=None, queue=None,
                  routing_key=None, version_type=None, record_to_index=None):
         """Initialize indexer.
@@ -138,7 +142,7 @@ class RecordIndexer(object):
         :param record_uuid: Record identifier.
         :param kwargs: Passed to :meth:`RecordIndexer.index`.
         """
-        return self.index(Record.get_record(record_uuid), **kwargs)
+        return self.index(self.record_cls.get_record(record_uuid), **kwargs)
 
     def delete(self, record, **kwargs):
         """Delete a record.
@@ -163,7 +167,7 @@ class RecordIndexer(object):
         :param record_uuid: Record identifier.
         :param kwargs: Passed to :meth:`RecordIndexer.delete`.
         """
-        self.delete(Record.get_record(record_uuid), **kwargs)
+        self.delete(self.record_cls.get_record(record_uuid), **kwargs)
 
     def bulk_index(self, record_id_iterator):
         """Bulk index records.
@@ -201,6 +205,10 @@ class RecordIndexer(object):
                 self._actionsiter(consumer.iterqueue()),
                 stats_only=True,
                 request_timeout=req_timeout,
+                expand_action_callback=(
+                    _es7_expand_action if ES_VERSION[0] >= 7
+                    else default_expand_action
+                ),
                 **es_bulk_kwargs
             )
 
@@ -269,7 +277,8 @@ class RecordIndexer(object):
         """
         index, doc_type = payload.get('index'), payload.get('doc_type')
         if not (index and doc_type):
-            record = Record.get_record(payload['id'], with_deleted=True)
+            record = self.record_cls.get_record(
+                payload['id'], with_deleted=True)
             index, doc_type = self.record_to_index(record)
         index, doc_type = self._prepare_index(index, doc_type)
 
@@ -280,14 +289,13 @@ class RecordIndexer(object):
             '_id': payload['id'],
         }
 
-    @es_bulk_param_compatibility
     def _index_action(self, payload):
         """Bulk index action.
 
         :param payload: Decoded message body.
         :returns: Dictionary defining an Elasticsearch bulk 'index' action.
         """
-        record = Record.get_record(payload['id'])
+        record = self.record_cls.get_record(payload['id'])
         index, doc_type = self.record_to_index(record)
 
         arguments = {}
